"use strict"; function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }





var _chunkD533HKNCcjs = require('../chunk-D533HKNC.cjs');

// src/cli/cli.ts
var _clipanion = require('clipanion');
var _module = require('module');
var _path = require('path'); var _path2 = _interopRequireDefault(_path);

// src/cli/helpers/base.command.ts

var BaseCommand = class extends _clipanion.Command {
  constructor() {
    super(...arguments);
    this.directory = _clipanion.Option.String("-D,--directory", _chunkD533HKNCcjs.sharedDefaultOptions.directory, {
      description: "Path to node_modules"
    });
    this.globFile = _clipanion.Option.String("-f,--glob-file", _chunkD533HKNCcjs.sharedDefaultOptions.globFile, {
      description: "Path to a custom glob file"
    });
    this.noDefaults = _clipanion.Option.Boolean("-n,--no-defaults", _chunkD533HKNCcjs.sharedDefaultOptions.noDefaults, {
      description: "Only includes/excludes globs specified by a custom glob file or CLI arguments"
    });
    this.globs = _clipanion.Option.Rest({ name: "globs" });
  }
};

// src/cli/commands/analyze.command.ts
var AnalyzeCommand = class extends BaseCommand {
  async execute() {
    const analyzeResults = await _chunkD533HKNCcjs.analyze.call(void 0, {
      globs: this.globs,
      noDefaults: this.noDefaults,
      globFile: this.globFile
    });
    console.log(JSON.stringify(analyzeResults, null, 2));
  }
};
AnalyzeCommand.paths = [["analyze"]];
AnalyzeCommand.usage = {
  description: "Helps determining why a file is included by the clean command without removing any files. Extra globs can be passed as positional args."
};

// src/cli/commands/clean.command.ts

var _prettybytes = require('pretty-bytes'); var _prettybytes2 = _interopRequireDefault(_prettybytes);
var _prettyms = require('pretty-ms'); var _prettyms2 = _interopRequireDefault(_prettyms);

// src/cli/utils/terminal.ts
var _readline = require('readline'); var _readline2 = _interopRequireDefault(_readline);
var _supportscolor = require('supports-color'); var _supportscolor2 = _interopRequireDefault(_supportscolor);
function makeLogger({ disabled }) {
  if (disabled) {
    const noop = () => void 0;
    return Object.keys(console).reduce((disabledConsole, key) => {
      disabledConsole[key] = noop;
      return disabledConsole;
    }, {});
  }
  return console;
}
async function yesOrNo(query) {
  const rl = _readline2.default.createInterface({
    input: process.stdin,
    output: process.stdout
  });
  return new Promise(
    (resolve) => rl.question(query, (answer) => {
      rl.close();
      resolve(/ye?s?/i.test(answer));
    })
  );
}
function colorizer(colorCode) {
  if (!_supportscolor2.default.stdout) {
    return (text) => String(text);
  }
  return (text) => `\x1B[${colorCode}m${text}\x1B[0m`;
}
var bold = colorizer("1");
var green = colorizer("32");
var yellow = colorizer("33");

// src/cli/commands/clean.command.ts
var CleanCommand = class extends BaseCommand {
  constructor() {
    super(...arguments);
    this.keepEmpty = _clipanion.Option.Boolean("-k,--keep-empty", false, {
      description: "Skips removing empty folders after removing contents"
    });
    this.dryRun = _clipanion.Option.Boolean("-d,--dry-run", false, {
      description: "Logs files that would be removed without removing any files"
    });
    this.silent = _clipanion.Option.Boolean("-s,--silent", false, {
      description: "Does not log anything to console (unless --json is enabled)"
    });
    this.yes = _clipanion.Option.Boolean("-y,--yes", false, {
      description: "Skips the confirmation prompt at the start of the script"
    });
    this.json = _clipanion.Option.Boolean("-j,--json", false, {
      description: "Output results as JSON"
    });
  }
  async execute() {
    const logger = makeLogger({ disabled: this.json || this.silent });
    logger.log(bold("clean-modules"), this.dryRun ? yellow("(dry run)") : "");
    if (!this.yes && !this.dryRun) {
      const warning = `
Preparing to clean node_modules at: ${this.directory}
Are you sure you want to continue? (Y/N) `;
      const confirmed = await yesOrNo(yellow(warning));
      if (!confirmed) {
        process.exit(0);
      }
    }
    logger.log("\nCleaning up node_modules...");
    const cleanupStart = (/* @__PURE__ */ new Date()).getTime();
    const { files, reducedSize, removedEmptyDirs } = await _chunkD533HKNCcjs.clean.call(void 0, {
      globs: this.globs,
      noDefaults: this.noDefaults,
      globFile: this.globFile,
      directory: this.directory,
      dryRun: this.dryRun
    });
    const cleanupDuration = (/* @__PURE__ */ new Date()).getTime() - cleanupStart;
    logger.log(green(`Done in ${_prettyms2.default.call(void 0, cleanupDuration)}!`));
    if (this.json) {
      const output = {
        removedFiles: files.length,
        reducedSize,
        removedEmptyDirs,
        duration: cleanupDuration,
        dryRun: this.dryRun
      };
      console.log(JSON.stringify(output, null, 2));
    } else {
      logger.log(bold("\nResults:"));
      logger.log("- size reduced:", green(_prettybytes2.default.call(void 0, reducedSize || 0)));
      logger.log("- files removed:", green(files.length));
      logger.log("- empty dirs removed:", green(removedEmptyDirs || 0));
    }
  }
};
CleanCommand.paths = [["clean"], _clipanion.Command.Default];
CleanCommand.usage = {
  description: "Default command. Removes unnecessary files to reduce the size of your node_modules directory. Extra globs can be passed as positional args."
};

// src/cli/cli.ts
var [_node, _app, ...args] = process.argv;
var esmRequire = _module.createRequire.call(void 0, _chunkD533HKNCcjs.importMetaUrl);
var cliDir = _chunkD533HKNCcjs.fileDir.call(void 0, _chunkD533HKNCcjs.importMetaUrl);
var { name, version } = esmRequire(_path2.default.resolve(cliDir, "..", "..", "package.json"));
var cli = new (0, _clipanion.Cli)({
  binaryLabel: name,
  binaryVersion: version,
  binaryName: name
});
cli.register(_clipanion.Builtins.HelpCommand);
cli.register(_clipanion.Builtins.VersionCommand);
cli.register(CleanCommand);
cli.register(AnalyzeCommand);
cli.runExit(args);
